-- MobMap - an ingame mob position database - v1.20
-- code file

-- coded 2007 by Slarti on EU-Blackhand


-- global constants

mobmap_shiftconst = {
	[0] = 1,
	[1] = 256,
	[2] = 65536,
	[3] = 1,
	[4] = 16777216,
	[5] = 4294967296,
	[6] = 1099511627776,
	[7] = 1,
}

mobmap_poweroftwo = {
	[0] = 1,
	[1] = 2,
	[2] = 4,
	[3] = 8,
	[4] = 16,
	[5] = 32,
	[6] = 64,
	[7] = 128,
	[8] = 256,
	[9] = 512,
	[10] = 1024,
	[11] = 2048,
	[12] = 4096,
	[13] = 8192,
	[14] = 16384,
	[15] = 32768,
	[16] = 65536,
	[17] = 131072,
	[18] = 262144,
	[19] = 524288,
	[20] = 1048576,
	[21] = 2097152,
	[22] = 4194304,
	[23] = 8388608,
	[24] = 16777216,
	[25] = 33554432,
	[26] = 67108864,
	[27] = 134217728,
	[28] = 268435456,
	[29] = 536870912,
	[30] = 1073741824,
	[31] = 2147483648,
	[32] = 4294967296,
	[33] = 8589934592,
	[34] = 17179869184,
	[35] = 34359738368,
	[36] = 68719476736,
	[37] = 137438953472,
	[38] = 274877906944,
	[39] = 549755813888,
	[40] = 1099511627776,
	[41] = 2199023255552,
	[42] = 4398046511104,
	[43] = 8796093022208,
	[44] = 17592186044416,
	[45] = 35184372088832,
	[46] = 70368744177664,
	[47] = 140737488355328,
	[48] = 281474976710656,
	[49] = 562949953421312,
	[50] = 1125899906842624,
	[51] = 2251799813685248,
	[52] = 4503599627370496,
}

MOBMAP_POSITION_DATABASE = 1;
MOBMAP_QUEST_DATABASE = 2;
MOBMAP_MERCHANT_DATABASE = 3;
MOBMAP_RECIPE_DATABASE = 4;
MOBMAP_ITEMNAME_HELPER_DATABASE = 5;
MOBMAP_MOBNAME_DATABASE = 6;

MOBMAP_DBVERSION = "0007";

MOBMAP_EXPORT_MAXSIZE = 180000;

MOBMAP_ISONAMAC = IsMacClient();

-- global variables

mobmap_enabled = true;
mobmap_scanning = true;
mobmap_minimap = false;
mobmap_positions = {};
mobmap_quests = {};
mobmap_merchantstock = {};
mobmap_tradeskills = {};
mobmap_loot = {};
mobmap_language = nil;
mobmap_dot_transparency = 0.7;
mobmap_dot_scale = 1.0;
mobmap_button_position = 1;
mobmap_display_database_loading_info = false;
mobmap_show_world_map_tooltips = true;
mobmap_dot_scaling_behavior = 0;

mobmap_disabled = false;

-- general database functions

function MobMap_GetMobCount()
	return mobmap_dbinfo["mobcount"];
end

function MobMap_GetQuestCount()
	return mobmap_dbinfo["questcount"];
end

function MobMap_GetMerchantCount()
	return mobmap_dbinfo["merchantcount"];
end

function MobMap_GetRecipeCount()
	return mobmap_dbinfo["recipecount"];
end

function MobMap_GetDBVersion()
	return mobmap_dbinfo["version"];	
end

function MobMap_GetDBBuildTime()
	return mobmap_dbinfo["buildtime"];	
end

function MobMap_GetDBLanguage()
	return mobmap_dbinfo["language"];	
end

-- quest objective parsing

mobmap_questsearch=false;

function MobMap_ParseQuestObjective(objective)
	MobMap_LoadDatabase(MOBMAP_POSITION_DATABASE);
	local filtered=string.match(objective, ".*:");
	if(filtered~=nil) then
		filtered=string.sub(filtered,1,string.len(filtered)-1);
	else
		filtered=objective;
	end
	parts={};
	partcount=0;
	for w in string.gmatch(filtered, "%S+") do
    		parts[partcount]=w;
	    	partcount=partcount+1;
	end
	local startingpoint, length, i;
	for startingpoint=0,partcount-1,1 do
		for length=partcount-startingpoint,1,-1 do
			local str="";
			for i=1,length,1 do
				str=str..parts[i+startingpoint-1];
				if(i<length) then str=str.." "; end
			end
			local mobid=MobMap_GetIDForMobName(str);
			if(mobid~=nil) then
				ShowUIPanel(MobMapFrame);
				MobMap_ShowPanel("MobMapMobSearchFrame");
				mobmap_questsearch=true;
				MobMap_ShowMobByName(str);
				return;
			end
		end
	end
	MobMap_DisplayMessage(MOBMAP_QUEST_PARSING_FAILED);
	ShowUIPanel(MobMapFrame);
end

function MobMap_ParseQuestTitle(questtitle)
	MobMap_LoadDatabase(MOBMAP_QUEST_DATABASE);
	local questid=MobMap_GetQuestIDByName(questtitle);
	if(questid==0) then
		MobMap_DisplayMessage(MOBMAP_QUEST_TITLE_NOT_FOUND);
	else
		MobMap_ShowQuestDetails(questid);
	end
end

function MobMap_ParseQuestTitleOrObjective(unknowntext)
	MobMap_LoadDatabase(MOBMAP_QUEST_DATABASE);
	local questid=MobMap_GetQuestIDByName(unknowntext);
	if(questid==0) then
		MobMap_ParseQuestObjective(unknowntext);
	else
		MobMap_ShowQuestDetails(questid);
	end
end

-- data acquisition

function MobMap_ScanTarget()
	local targetname=UnitName("target");
	if(targetname~=nil and targetname~=UNKNOWNOBJECT) then
		if(not UnitPlayerControlled("target") and not UnitIsPlayer("target") and not UnitIsDeadOrGhost("target") and (CheckInteractDistance("target", 1) or UnitReaction("target", "player")<5)) then
			if(not WorldMapFrame:IsVisible()) then
				local moblevel=UnitLevel("target");
				if(mobmap_positions[targetname]==nil) then
					mobmap_positions[targetname]={};
				end
				local x, y, zonename = MobMap_GetPlayerCoordinates();
				if(mobmap_positions[targetname][zonename]==nil) then
					mobmap_positions[targetname][zonename]={};
				end
				if(mobmap_positions[targetname]["min"]==nil) then
					mobmap_positions[targetname]["min"]=moblevel;
				else
					if(moblevel<mobmap_positions[targetname]["min"]) then mobmap_positions[targetname]["min"]=moblevel; end
				end
				if(mobmap_positions[targetname]["max"]==nil) then
					mobmap_positions[targetname]["max"]=moblevel;
				else
					if(moblevel>mobmap_positions[targetname]["max"]) then mobmap_positions[targetname]["max"]=moblevel; end
				end
				if(mobmap_positions[targetname][zonename][x.."/"..y]==nil) then
					mobmap_positions[targetname][zonename][x.."/"..y]=1;
					if(mobmap_debug) then MobMap_DisplayMessage("found "..targetname.." in zone "..zonename.." at "..x.." / "..y); end
				end
			end
		end
	end
end

function MobMap_ScanQuest()
	if(not QuestFrame:IsVisible()) then return; end
	local title=GetTitleText();
	if(title~=nil) then
		local filteredtitle=string.match(title,".*%] (.*)");
		if(filteredtitle~=nil) then
			title=filteredtitle;
		end
	end
	local objective=GetObjectiveText();
	local npcname=QuestFrameNpcNameText:GetText();
	local zonename=GetRealZoneText();
	local playerfaction=UnitFactionGroup("player");
	if(title==nil or objective==nil or npcname==nil) then return; end
	local choiceRewards={};
	local alwaysRewards={};
	local i=0;
	while(GetQuestItemLink("choice",i+1)~=nil) do
		i=i+1;
		local link=GetQuestItemLink("choice",i);
		local id,suffix=MobMap_GetItemIDFromItemLink(link);
		choiceRewards["reward"..i]=id.."|"..suffix;
	end

	i=0;
	while(GetQuestItemLink("reward",i+1)~=nil) do
		i=i+1;
		local link=GetQuestItemLink("reward",i);
		local id,suffix=MobMap_GetItemIDFromItemLink(link);
		alwaysRewards["reward"..i]=id.."|"..suffix;
	end

	local completetitle=title;
	local counter=0;
	while(mobmap_quests[completetitle]~=nil) do
		if(mobmap_quests[completetitle].obj==objective) then
			return;
		end
		counter=counter+1;
		completetitle=title.."|"..counter;
	end
	mobmap_quests[completetitle]={obj=objective, level=0, npc=npcname, zone=zonename, group="solo", choice=choiceRewards, always=alwaysRewards, faction=playerfaction};
end

function MobMap_ScanQuestLog()
	local i=0;
	while(GetQuestLogTitle(i+1)~=nil) do
		i=i+1;
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete = GetQuestLogTitle(i);
		if(questTitle~=nil) then
			local filteredtitle=string.match(questTitle,".*%] (.*)");
			if(filteredtitle~=nil) then
				questTitle=filteredtitle;
			end
		end
		local unlocalizedQuestTag="solo";
		if(questTag==ELITE) then unlocalizedQuestTag="elite"; end
		if(questTag==LFG_TYPE_DUNGEON) then unlocalizedQuestTag="dungeon"; end
		if(questTag==PVP) then unlocalizedQuestTag="pvp"; end
		if(questTag==RAID) then unlocalizedQuestTag="raid"; end
		if(questTag==GROUP) then unlocalizedQuestTag="group"; end
		if(not isHeader) then
			local completetitle=questTitle;
			local counter=0;
			while(mobmap_quests[completetitle]~=nil) do
				if(mobmap_quests[completetitle].level==0) then
					SelectQuestLogEntry(i);
					local questDescription, questObjectives = GetQuestLogQuestText();
					if(questObjectives==mobmap_quests[completetitle].obj) then
						mobmap_quests[completetitle].level=level;
						mobmap_quests[completetitle].group=unlocalizedQuestTag;
					end
				end
				counter=counter+1;
				completetitle=questTitle.."|"..counter;
			end
		end
	end
end

function MobMap_ScanMerchant()
	if(not MerchantFrame:IsVisible()) then return; end
	local i;
	local merchantName=MerchantNameText:GetText();
	local merchantItemCount=GetMerchantNumItems();
	local merchantData={};

	local item;
	for item=1, merchantItemCount, 1 do
		local itemData={};
		local itemlink=GetMerchantItemLink(item);
		if(itemlink~=nil) then
			local itemid=MobMap_GetItemIDFromItemLink(itemlink);
			local name, texture, price, quantity, numAvailable, isUsable, extendedCost = GetMerchantItemInfo(item);
			itemData["id"]=itemid;
			itemData["price"]=price;
			itemData["quantity"]=quantity;
			if(numAvailable~=-1) then
				itemData["limited"]=1;
			else
				itemData["limited"]=0;
			end

			if(extendedCost) then
				local honorPoints, arenaPoints, itemCount = GetMerchantItemCostInfo(item);
				itemData["honorprice"]=honorPoints;
				itemData["arenaprice"]=arenaPoints;
				local token;
				for token=1, itemCount, 1 do
					local itemTexture, itemValue = GetMerchantItemCostItem(item, token);
					MobMapScanTooltip:Show();
					MobMapScanTooltip:SetOwner(UIParent, "ANCHOR_NONE");
					MobMapScanTooltip:SetMerchantCostItem(item, token);
					local itemName=MobMapScanTooltipTextLeft1:GetText();
					MobMapScanTooltip:Hide();
					if(itemName) then
						itemData["token"..token]=itemName.."|"..itemTexture.."|"..itemValue;
					end
				end
			end

			merchantData[tostring(item)]=itemData;
		end
	end

	mobmap_merchantstock[merchantName]=merchantData;
end

mobmap_original_GetTradeSkillInfo=nil;
mobmap_last_tradeskill_scan=0;

function MobMap_HookGetTradeSkillInfo()
	mobmap_original_GetTradeSkillInfo=GetTradeSkillInfo;
	GetTradeSkillInfo=MobMap_GetTradeSkillInfo;
end

function MobMap_GetTradeSkillInfo(skillIndex)
	local skillName, skillType, numAvailable, isExpanded = mobmap_original_GetTradeSkillInfo(skillIndex);

	if(skillName~=nil and skillType~="header" and mobmap_tradeskills[skillName]==nil) then
		if(mobmap_last_tradeskill_scan+2.0<GetTime()) then
			mobmap_last_tradeskill_scan=GetTime();
			local tradeskillItem={};
			if(mobmap_tradeskills[skillName]~=nil) then
				tradeskillItem=mobmap_tradeskills[skillName];
			end

			local tradeskillName, tradeskillLevel = GetTradeSkillLine();
			
			if(tradeskillName~=nil) then
				tradeskillItem["category"]=tradeskillName;
				local itemlink=GetTradeSkillItemLink(skillIndex);
				local errors=false;
				if(itemlink~=nil) then
					local reagentCount=GetTradeSkillNumReagents(skillIndex);			
					local i;
					for i=1,reagentCount,1 do
						local currentReagent={};
						local reagentName, reagentTexture, reagentCount = GetTradeSkillReagentInfo(skillIndex, i);
						local link=GetTradeSkillReagentItemLink(skillIndex, i);
						if(link==nil) then
							errors=true;
							break;
						end
						local itemid=MobMap_GetItemIDFromItemLink(link);
						currentReagent.itemid=itemid;
						currentReagent.count=reagentCount;
						tradeskillItem["reagent"..i]=currentReagent;
					end
					local minMade, maxMade = GetTradeSkillNumMade(skillIndex);
					local itemid=MobMap_GetItemIDFromItemLink(itemlink);
					tradeskillItem["itemid"]=itemid;
					tradeskillItem["level"]=tradeskillLevel;
					if(minMade~=1 or maxMade~=1) then
						tradeskillItem["minmade"]=minMade;
						tradeskillItem["maxmade"]=maxMade;
					end

					if(errors==false) then 
						mobmap_tradeskills[skillName]=tradeskillItem;
					end
				end
			end
		end
	end

	return skillName, skillType, numAvailable, isExpanded;
end

mobmap_original_GetCraftInfo=nil;

function MobMap_HookGetCraftInfo()
	mobmap_original_GetCraftInfo=GetCraftInfo;
	GetCraftInfo=MobMap_GetCraftInfo;
end

function MobMap_GetCraftInfo(skillIndex)
	local craftName, craftSubSpellName, craftType, numAvailable, isExpanded, trainingPointCost, requiredLevel = mobmap_original_GetCraftInfo(skillIndex);

	if(craftName~=nil and craftType~="header" and mobmap_tradeskills[craftName]==nil) then
		if(mobmap_last_tradeskill_scan+2.0<GetTime()) then
			mobmap_last_tradeskill_scan=GetTime();
			local tradeskillItem={};
			local tradeskillName, tradeskillLevel = GetCraftDisplaySkillLine();
			
			if(tradeskillName~=nil) then
				tradeskillItem["category"]=tradeskillName;
				local enchantlink=GetCraftItemLink(skillIndex);
				local errors=false;
				if(enchantlink~=nil) then
					local reagentCount=GetCraftNumReagents(skillIndex);
					local i;
					for i=1,reagentCount,1 do
						local currentReagent={};
						local reagentName, reagentTexture, reagentCount = GetCraftReagentInfo(skillIndex, i);
						local link=GetCraftReagentItemLink(skillIndex, i);
						if(link==nil) then
							errors=true;
							break;
						end
						local itemid=MobMap_GetItemIDFromItemLink(link);
						currentReagent.itemid=itemid;
						currentReagent.count=reagentCount;
						tradeskillItem["reagent"..i]=currentReagent;
					end
					local enchantid=MobMap_GetEnchantIDFromEnchantLink(enchantlink);
					tradeskillItem["enchantid"]=enchantid;
					tradeskillItem["level"]=tradeskillLevel;

					if(errors==false) then mobmap_tradeskills[craftName]=tradeskillItem; end
				end
			end
		end
	end

	return craftName, craftSubSpellName, craftType, numAvailable, isExpanded, trainingPointCost, requiredLevel;
end

function MobMap_ScanLoot()
	local mobname=nil;
	if(IsFishingLoot()) then
		mobname="FISH";
	else
		local targetname=UnitName("target");
		if(targetname~=nil and targetname~=UNKNOWNOBJECT) then
			if(CheckInteractDistance("target",4) and UnitIsDead("target") and not UnitIsFriend("player", "target")) then
				mobname=UnitName("target");
			else
				mobname="UNKN";
			end
		else
			mobname="UNKN";
		end
	end
	local x, y, zonename = MobMap_GetPlayerCoordinates();
	local difficulty=GetInstanceDifficulty();
	if(difficulty==2) then zonename=zonename.."HEROIC"; end
	local combinedname=mobname.."#"..x.."#"..y.."#"..zonename;

	local lootstring="+";
	local i;
	for i=1,GetNumLootItems(),1 do
		if(LootSlotIsItem(i)) then
			local itemid, suffixid, uniqueid = MobMap_GetItemIDFromItemLink(GetLootSlotLink(i));
			if(not(tonumber(suffixid)<0)) then
				uniqueid=0;
			end
			local _, _, count = GetLootSlotInfo(1);
			lootstring=lootstring..itemid..":"..suffixid..":"..uniqueid..":"..count.."#";
		elseif(LootSlotIsCoin(i)) then
			local _, moneystring = GetLootSlotInfo(i);
			local money=MobMap_GetMoneyFromMoneyString(moneystring);
			lootstring=lootstring..money.."#";
		end
	end
	if(lootstring=="+") then return; end

	if(mobmap_loot[combinedname]~=nil) then
		mobmap_loot[combinedname]=mobmap_loot[combinedname]..lootstring;
	else
		mobmap_loot[combinedname]=lootstring;
	end
end

--- user interface

function MobMapButtonFrame_OnLoad()
	SLASH_MOBMAP1 = "/mobmap";
	SlashCmdList["MOBMAP"] = MobMap_Command;
end

function MobMap_PlaceMobMapButtonFrame()
	if(mobmap_button_position==0) then
		MobMapButtonFrame:Hide();
	end
	if(mobmap_button_position==1) then
		MobMapButtonFrame:SetParent(WorldMapPositioningGuide);
		MobMapButtonFrame:SetPoint("TOPLEFT","WorldMapPositioningGuide","TOPLEFT",10,-24);
		MobMapButtonFrame:SetFrameLevel(WorldMapPositioningGuide:GetFrameLevel()+20);
		MobMapButton:ClearAllPoints();
		MobMapButton:SetPoint("TOPLEFT","MobMapButtonFrame","TOPLEFT",0,0);
		MobMapCheckButton:ClearAllPoints();
		MobMapCheckButton:SetPoint("TOPLEFT","MobMapButtonFrame","TOPLEFT",0,-22);
		MobMapButtonFrameCurrentMob:ClearAllPoints();
		MobMapButtonFrameCurrentMob:SetPoint("TOPLEFT","MobMapButtonFrame","TOPLEFT",106,-6);
		MobMapButtonFrameCurrentMob:SetJustifyH("left");
		MobMapCheckButton:SetChecked(mobmap_enabled);
		MobMapButtonFrame:Show();
	end
	if(mobmap_button_position==2) then
		MobMapButtonFrame:SetParent(WorldMapPositioningGuide);
		MobMapButtonFrame:SetPoint("TOPRIGHT","WorldMapPositioningGuide","TOPRIGHT",-4,-24);
		MobMapButtonFrame:SetFrameLevel(WorldMapPositioningGuide:GetFrameLevel()+20);
		MobMapButton:ClearAllPoints();
		MobMapButton:SetPoint("TOPRIGHT","MobMapButtonFrame","TOPRIGHT",0,0);
		MobMapCheckButton:ClearAllPoints();
		MobMapCheckButton:SetPoint("TOPRIGHT","MobMapButtonFrame","TOPRIGHT",-170,-22);
		MobMapButtonFrameCurrentMob:ClearAllPoints();
		MobMapButtonFrameCurrentMob:SetPoint("TOPRIGHT","MobMapButtonFrame","TOPRIGHT",-106,-6);
		MobMapButtonFrameCurrentMob:SetJustifyH("right");
		MobMapCheckButton:SetChecked(mobmap_enabled);
		MobMapButtonFrame:Show();
	end
	if(mobmap_button_position==3) then
		MobMapButtonFrame:SetParent(WorldMapPositioningGuide);
		MobMapButtonFrame:SetPoint("BOTTOMLEFT","WorldMapPositioningGuide","BOTTOMLEFT",0,4);
		MobMapButtonFrame:SetFrameLevel(WorldMapPositioningGuide:GetFrameLevel()+20);
		MobMapButton:ClearAllPoints();
		MobMapButton:SetPoint("BOTTOMLEFT","MobMapButtonFrame","BOTTOMLEFT",4,0);
		MobMapCheckButton:ClearAllPoints();
		MobMapCheckButton:SetPoint("LEFT","MobMapButton","RIGHT",10,0);
		MobMapButtonFrameCurrentMob:ClearAllPoints();
		MobMapButtonFrameCurrentMob:SetPoint("LEFT","MobMapCheckButton","RIGHT",200,0);
		MobMapButtonFrameCurrentMob:SetJustifyH("left");
		MobMapCheckButton:SetChecked(mobmap_enabled);
		MobMapButtonFrame:Show();
	end
end

function MobMap_ShowMobMapFrame()
	if(mobmap_disabled) then return; end
	if(not MobMapFrame:IsVisible()) then
		ShowUIPanel(MobMapFrame);
		if(MobMapMobSearchFrame) then MobMap_UpdateMobMapFrame(); end
	end
end

function MobMap_ToggleMobMapFrame()
	if(mobmap_disabled) then return; end
	if(MobMapFrame:IsVisible()) then
		HideUIPanel(MobMapFrame);
	else
		ShowUIPanel(MobMapFrame);
		if(MobMapMobSearchFrame) then MobMap_UpdateMobMapFrame(); end
	end
end

function MobMap_ShowExportFrame()
	if(MobMapExportFrame:IsVisible()) then
		HideUIPanel(MobMapExportFrame);
	else
		if(GetLocale()~="deDE" and GetLocale()~="enUS" and GetLocale()~="enGB") then
			MobMap_DisplayMessage(MOBMAP_UNSUPPORTED_LOCALE);
		else
			MobMap_ExportData();
		end
	end
end

function MobMapFrame_OnLoad()
	this:RegisterEvent("WORLD_MAP_UPDATE");
	this:RegisterEvent("QUEST_DETAIL");
	this:RegisterEvent("QUEST_LOG_UPDATE");
	this:RegisterEvent("PLAYER_TARGET_CHANGED");
	this:RegisterEvent("MERCHANT_SHOW");
	this:RegisterEvent("MERCHANT_UPDATE");
	this:RegisterEvent("ADDON_LOADED");
	this:RegisterEvent("LOOT_OPENED");
	local addonsAvailable=true;
	local name, _, _, _, loadable = GetAddOnInfo("MobMapDatabaseStub1");
	if(name==nil or loadable==nil) then addonsAvailable=false; end
	name, _, _, _, loadable = GetAddOnInfo("MobMapDatabaseStub2");
	if(name==nil or loadable==nil) then addonsAvailable=false; end
	name, _, _, _, loadable = GetAddOnInfo("MobMapDatabaseStub3");
	if(name==nil or loadable==nil) then addonsAvailable=false; end
	name, _, _, _, loadable = GetAddOnInfo("MobMapDatabaseStub4");
	if(name==nil or loadable==nil) then addonsAvailable=false; end
	name, _, _, _, loadable = GetAddOnInfo("MobMapDatabaseStub5");
	if(name==nil or loadable==nil) then addonsAvailable=false; end
	name, _, _, _, loadable = GetAddOnInfo("MobMapDatabaseStub6");
	if(name==nil or loadable==nil) then addonsAvailable=false; end

	if(addonsAvailable==false) then
		MobMap_DisplayMessage(MOBMAP_ERROR_ON_LOAD);
		mobmap_disabled=true;
	else
		if(mobmap_dbinfo==nil) then
			MobMap_ErrorMessage(MOBMAP_ERROR_NO_DB_INSTALLED);
			mobmap_disabled=true;
		else
			if(MobMap_GetDBVersion()~=MOBMAP_DBVERSION) then
				MobMap_ErrorMessage(MOBMAP_ERROR_INCOMPATIBLE_DB);
				mobmap_disabled=true;
			else
				MobMap_DisplayMessage(MOBMAP_VERSION.." loaded");
			end
		end
	end


	if(not mobmap_disabled) then
		SLASH_MOBMAP1 = "/mobmap";
		SlashCmdList["MOBMAP"] = MobMap_Command;
	else
		MobMapButtonFrame:Hide();
	end

	if(mobmap_scanning==true and (GetLocale()~="deDE" and GetLocale()~="enUS" and GetLocale()~="enGB")) then
		mobmap_scanning=false;
	end

	if(mobmap_scanning==true) then
		MobMap_HookGetTradeSkillInfo();
		MobMap_HookGetCraftInfo();
	end
end

mobmap_lastzone="";

function MobMapFrame_OnEvent()
	if(event=="WORLD_MAP_UPDATE") then
		if(mobmap_enabled and WorldMapFrame:IsVisible() and MobMapMobSearchFrame) then
			if(mobmap_lastzone~=MobMap_GetCurrentMapZoneName()) then
				mobmap_lastzone=MobMap_GetCurrentMapZoneName();
				MobMap_Display();
			end
		end
	elseif(event=="PLAYER_TARGET_CHANGED") then
		if(mobmap_scanning) then 
			MobMap_ScanTarget();
		else
			MobMapFrame:UnregisterEvent("PLAYER_TARGET_CHANGED");
		end
	elseif(event=="LOOT_OPENED") then
		if(mobmap_scanning) then
			MobMap_ScanLoot();
		else
			MobMapFrame:UnregisterEvent("LOOT_OPENED");
		end
	elseif(event=="QUEST_DETAIL") then
		if(mobmap_scanning) then 
			MobMap_ScanQuest();
		end
	elseif(event=="QUEST_LOG_UPDATE") then
		if(mobmap_scanning) then 
			MobMap_ScanQuestLog();
		end
	elseif(event=="MERCHANT_SHOW") then
		if(mobmap_scanning) then 
			MobMap_ScanMerchant();
		end
	elseif(event=="MERCHANT_UPDATE") then
		if(mobmap_scanning) then 
			MobMap_ScanMerchant();
		end
	elseif(event=="ADDON_LOADED") then
		if(string.find(arg1,"MobMap")~=nil) then
			if(mobmap_scanning==true) then
				local k,v;
				local count=0;
				for k,v in pairs(mobmap_positions) do
					count=count+1;
				end
				if(count>2000) then
					MobMap_DisplayMessage(MOBMAP_DATA_SIZE_WARNING);
				end
				if(mobmap_language~=nil and GetLocale()~=mobmap_language) then
					MobMap_DeletePositionData();
				end
			end
			mobmap_language=GetLocale();
			if(not mobmap_disabled) then
				if(mobmap_alternate_button_position==true) then
					mobmap_button_position=2;
				end
				MobMap_PlaceMobMapButtonFrame();
			end
			MobMapFrame:UnregisterEvent("ADDON_LOADED");
		end
	end
end

function MobMap_SetMapToZone(zonename)
	local continentnames={GetMapContinents()};
	for k,v in pairs(continentnames) do
		local zonenames={GetMapZones(k)};
		for x,y in pairs(zonenames) do
			if(y==zonename) then
				SetMapZoom(k,x);
				return true;
			end
		end
	end
	return false;
end

function MobMap_GetCurrentMapZoneName()
	local zonenames={GetMapZones(GetCurrentMapContinent())};
	return zonenames[GetCurrentMapZone()];
end

mobmap_timer=0;
gatherer_astrolabe_library=nil;
gatherer_astrolabe_library_minimap_update_time=0.2;
gatherer_astrolabe_library_time_changed=false;

mobmap_astrolabe_library_minimap_update_time=0.2;
mobmap_astrolabe_library_time_changed=false;

function MobMapTimerFrame_OnUpdate()
	mobmap_timer=mobmap_timer-arg1;
	if(mobmap_timer<=0) then
		MobMapQuestWatchButtons_Update();
		MobMapQuestLogButtons_Update();
		-- The following hack is necessary to increase the update interval of the Astrolabe library used by Gatherer.
		-- Astrolabe switches the zone displayed on the world map to the current zone and back to the original again
		-- to determine the player position which causes MobMap to redraw after every change which makes WoW stutter.
		-- Astrolabe's Update Interval is being changed as long as the world map is open and is reset to the original
		-- value afterwards
		if(Gatherer~=nil and Gatherer.AstrolabeVersion=="Astrolabe-0.3") then
			if(gatherer_astrolabe_library==nil) then gatherer_astrolabe_library=DongleStub(Gatherer.AstrolabeVersion); end
			if(gatherer_astrolabe_library_time_changed and not WorldMapFrame:IsShown()) then
				gatherer_astrolabe_library.MinimapUpdateTime=gatherer_astrolabe_library_minimap_update_time;
				gatherer_astrolabe_library_time_changed=false;
			elseif(not gatherer_astrolabe_library_time_changed and WorldMapFrame:IsShown()) then
				gatherer_astrolabe_library_minimap_update_time=gatherer_astrolabe_library.MinimapUpdateTime;
				gatherer_astrolabe_library.MinimapUpdateTime=5.0;
				gatherer_astrolabe_library_time_changed=true;
			end
		end
		if(Gatherer~=nil and Gatherer.AstrolabeVersion=="Astrolabe-0.2") then
			if(gatherer_astrolabe_library==nil) then gatherer_astrolabe_library=AceLibrary:GetInstance(Gatherer.AstrolabeVersion); end
			if(gatherer_astrolabe_library_time_changed and not WorldMapFrame:IsShown()) then
				gatherer_astrolabe_library.MinimapUpdateTime=gatherer_astrolabe_library_minimap_update_time;
				gatherer_astrolabe_library_time_changed=false;
			elseif(not gatherer_astrolabe_library_time_changed and WorldMapFrame:IsShown()) then
				gatherer_astrolabe_library_minimap_update_time=gatherer_astrolabe_library.MinimapUpdateTime;
				gatherer_astrolabe_library.MinimapUpdateTime=5.0;
				gatherer_astrolabe_library_time_changed=true;
			end
		end

		-- do the same for the mobmap astrolabe library, if it's not the same as the gatherer one (and if it's loaded, of course!)
		if(mobmap_astrolabe_version and ((Gatherer~=nil and Gatherer.AstrolabeVersion~=mobmap_astrolabe_version) or Gatherer==nil)) then
			if(mobmap_astrolabe_library==nil) then mobmap_astrolabe_library=DongleStub(mobmap_astrolabe_version); end
			if(mobmap_astrolabe_library_time_changed and not WorldMapFrame:IsShown()) then
				mobmap_astrolabe_library.MinimapUpdateTime=mobmap_astrolabe_library_minimap_update_time;
				mobmap_astrolabe_library_time_changed=false;
			elseif(not mobmap_astrolabe_library_time_changed and WorldMapFrame:IsShown()) then
				mobmap_astrolabe_library_minimap_update_time=mobmap_astrolabe_library.MinimapUpdateTime;
				mobmap_astrolabe_library.MinimapUpdateTime=5.0;
				mobmap_astrolabe_library_time_changed=true;
			end
		end

		-- update MobMap minimap icons
		if(mobmap_astrolabe_version and mobmap_minimap==true) then
			MobMap_UpdateMinimapIcons();
		end

		mobmap_timer=0.2;
	end
end

function MobMap_ShowPanel(panel)
	if(MobMapMobSearchFrame~=nil) then MobMapMobSearchFrame:Hide(); end
	if(MobMapQuestListFrame~=nil) then MobMapQuestListFrame:Hide(); end
	if(MobMapMerchantListFrame~=nil) then MobMapMerchantListFrame:Hide(); end
	if(MobMapRecipeListFrame~=nil) then MobMapRecipeListFrame:Hide(); end
	if(MobMapOptionsFrame~=nil) then MobMapOptionsFrame:Hide(); end
	if(MobMapAboutFrame~=nil) then MobMapAboutFrame:Hide(); end
	PanelTemplates_DeselectTab(MobMapSearchModeButton);
	PanelTemplates_DeselectTab(MobMapQuestModeButton);
	PanelTemplates_DeselectTab(MobMapMerchantModeButton);
	PanelTemplates_DeselectTab(MobMapRecipeModeButton);
	PanelTemplates_DeselectTab(MobMapOptionsModeButton);
	PanelTemplates_DeselectTab(MobMapAboutModeButton);
	if(panel=="MobMapMobSearchFrame") then
		MobMap_LoadDatabase(MOBMAP_POSITION_DATABASE);
		PanelTemplates_SelectTab(MobMapSearchModeButton);
		MobMapMobSearchFrame:Show();
		MobMap_UpdateMobMapFrame();
		return;
	end

	if(panel=="MobMapQuestListFrame") then
		MobMap_LoadDatabase(MOBMAP_QUEST_DATABASE);
		MobMapQuestListFrame:Show();
		PanelTemplates_SelectTab(MobMapQuestModeButton);
		if(mobmap_questlist==nil) then
			MobMap_RefreshQuestList();
		end
		return;
	end

	if(panel=="MobMapMerchantListFrame") then
		MobMap_LoadDatabase(MOBMAP_MERCHANT_DATABASE);
		MobMapMerchantListFrame:Show();
		PanelTemplates_SelectTab(MobMapMerchantModeButton);
		if(mobmap_merchantlist==nil) then
			MobMap_RefreshMerchantList();
		end
		return;
	end

	if(panel=="MobMapRecipeListFrame") then
		MobMap_LoadDatabase(MOBMAP_RECIPE_DATABASE);
		MobMapRecipeListFrame:Show();
		PanelTemplates_SelectTab(MobMapRecipeModeButton);
		if(mobmap_recipelist==nil) then
			MobMap_RefreshRecipeList();
		end
		return;
	end

	if(panel=="MobMapOptionsFrame") then
		PanelTemplates_SelectTab(MobMapOptionsModeButton);
		MobMapOptionsFrame:Show();
		return;
	end

	if(panel=="MobMapAboutFrame") then
		PanelTemplates_SelectTab(MobMapAboutModeButton);
		MobMapAboutFrame:Show();
		return;
	end
end

function MobMapQuestWatchButtons_Update()
	if(mobmap_disabled) then return; end
	if(QuestWatchFrame:IsVisible()) then		
		local i;
		for i=1,30,1 do
			local frame=getglobal("QuestWatchLine"..i);
			local button=getglobal("MobMapQuestWatchButton"..i);
			if(frame:IsVisible()) then
				local questobj=frame:GetText();
				if(questobj) then
					if(button==nil) then
						button=CreateFrame("Frame","MobMapQuestWatchButton"..i,QuestWatchFrame,"MobMapQuestButtonFrameTemplate");
						button:ClearAllPoints();
						button:SetPoint("RIGHT","QuestWatchLine"..i,"LEFT",0,0);
						button:SetAlpha(0.6);
					end
					if(string.sub(questobj,1,3)==" - ") then
						button.questobj=string.sub(questobj,4);
						button.questtitle=nil;
						button.unknowntext=nil;
						button:Show();
					else
						local title=questobj;
						if(title~=nil) then
							local filteredtitle=string.match(title,".*%] (.*)");
							if(filteredtitle~=nil) then
								title=filteredtitle;
							end
						end
						button.questtitle=title;
						button.questobj=nil;
						button.unknowntext=nil;
						button:Show();
					end
				end
			else
				if(button~=nil) then button:Hide(); end
			end
		end
	end
	if(EQL3_QuestWatchFrame~=nil and EQL3_QuestWatchFrame:IsVisible()) then
		local i;
		for i=1,30,1 do
			local frame=getglobal("EQL3_QuestWatchLine"..i);
			local button=getglobal("MobMapELQ3QuestWatchButton"..i);
			if(frame:IsVisible()) then
				local questobj=frame:GetText();
				if(questobj) then
					if(button==nil) then
						button=CreateFrame("Frame","MobMapELQ3QuestWatchButton"..i,EQL3_QuestWatchFrame,"MobMapQuestButtonFrameTemplate");
						button:ClearAllPoints();
						button:SetPoint("RIGHT","EQL3_QuestWatchLine"..i,"LEFT",0,0);
						button:SetAlpha(0.6);
					end
					if(string.find(questobj, "^Quest Tracker.*")==nil) then
						local filteredtitle=string.match(questobj,".*%] (.*)");
						if(filteredtitle~=nil) then
							questobj=filteredtitle;
						end
						button.questobj=nil;
						button.questtitle=nil;
						button.unknowntext=questobj;
						button:Show();
					else
						button:Hide();
					end
				end
			else
				if(button~=nil) then button:Hide(); end
			end
		end
	end
end

function MobMapQuestWatchButtons_HideAll()
	local i;
	for i=1,30,1 do
		local frame=getglobal("MobMapQuestWatchButton"..i);
		if(frame~=nil) then frame:Hide(); end
	end
	for i=1,30,1 do
		local frame=getglobal("MobMapELQ3QuestWatchButton"..i);
		if(frame~=nil) then frame:Hide(); end
	end
end

function MobMapQuestLogButtons_Update()
	if(mobmap_disabled) then return; end
	if(QuestLogFrame:IsVisible()) then		
		local frame=getglobal("QuestLogQuestTitle");
		local button=getglobal("MobMapQuestLogTitleButton");
		if(button==nil) then
			button=CreateFrame("Frame","MobMapQuestLogTitleButton",QuestLogDetailScrollChildFrame,"MobMapQuestButtonFrameTemplate");
			button:ClearAllPoints();
			button:SetFrameStrata("MEDIUM");
			button:SetPoint("LEFT","QuestLogQuestTitle","RIGHT",-8,0);
			button:SetAlpha(0.6);
		end
		local title=frame:GetText();
		if(title~=nil) then
			local filteredtitle=string.match(title,".*%] (.*)");
			if(filteredtitle~=nil) then
				title=filteredtitle;
			end
		end
		button.questtitle=title;
		button:Show();

		local i;
		for i=1,10,1 do
			frame=getglobal("QuestLogObjective"..i);
			button=getglobal("MobMapQuestLogButton"..i);
			if(frame:IsVisible()) then
				local questobj=frame:GetText();
				if(questobj) then
					if(button==nil) then
						button=CreateFrame("Frame","MobMapQuestLogButton"..i,QuestLogDetailScrollChildFrame,"MobMapQuestButtonFrameTemplate");
						button:ClearAllPoints();
						button:SetFrameStrata("MEDIUM");
						button:SetPoint("LEFT","QuestLogObjective"..i,"RIGHT",-16,0);
						button:SetAlpha(0.6);
					end
					button.questobj=questobj;
					button:Show();
				end
			else
				if(button~=nil) then button:Hide(); end
			end
		end
	end

	if(EQL3_QuestLogFrame~=nil and EQL3_QuestLogFrame:IsVisible()) then		
		local frame=getglobal("EQL3_QuestLogQuestTitle");
		local button=getglobal("MobMapELQ3QuestLogTitleButton");
		if(button==nil) then
			button=CreateFrame("Frame","MobMapELQ3QuestLogTitleButton",EQL3_QuestLogDetailScrollChildFrame,"MobMapQuestButtonFrameTemplate");
			button:ClearAllPoints();
			button:SetFrameStrata("MEDIUM");
			button:SetPoint("LEFT","EQL3_QuestLogQuestTitle","RIGHT",-8,0);
			button:SetAlpha(0.6);
		end
		local title=frame:GetText();
		if(title~=nil) then
			local filteredtitle=string.match(title,".*%] (.*)");
			if(filteredtitle~=nil) then
				title=filteredtitle;
			end
		end
		button.questtitle=title;
		button:Show();

		local i;
		for i=1,10,1 do
			frame=getglobal("EQL3_QuestLogObjective"..i);
			button=getglobal("MobMapELQ3QuestLogButton"..i);
			if(frame:IsVisible()) then
				local questobj=frame:GetText();
				if(questobj) then
					if(button==nil) then
						button=CreateFrame("Frame","MobMapELQ3QuestLogButton"..i,EQL3_QuestLogDetailScrollChildFrame,"MobMapQuestButtonFrameTemplate");
						button:ClearAllPoints();
						button:SetFrameStrata("MEDIUM");
						button:SetPoint("LEFT","EQL3_QuestLogObjective"..i,"RIGHT",-16,0);
						button:SetAlpha(0.6);
					end
					button.questobj=questobj;
					button:Show();
				end
			else
				if(button~=nil) then button:Hide(); end
			end
		end
	end
	
	if(UberQuest_Details~=nil and UberQuest_Details:IsVisible()) then		
		local frame=getglobal("UberQuest_Details_ScrollChild_QuestTitle");
		local button=getglobal("MobMapUberQuestQuestLogTitleButton");
		if(button==nil) then
			button=CreateFrame("Frame","MobMapUberQuestQuestLogTitleButton",UberQuest_Details_ScrollChild,"MobMapQuestButtonFrameTemplate");
			button:ClearAllPoints();
			button:SetFrameStrata("MEDIUM");
			button:SetPoint("LEFT","UberQuest_Details_ScrollChild_QuestTitle","RIGHT",-8,0);
			button:SetAlpha(0.6);
		end
		local title=frame:GetText();
		if(title~=nil) then
			local filteredtitle=string.match(title,".*%] (.*)");
			if(filteredtitle~=nil) then
				title=filteredtitle;
			end
		end
		button.questtitle=title;
		button:Show();

		local i;
		for i=1,10,1 do
			frame=getglobal("UberQuest_Details_ScrollChild_Objective"..i);
			button=getglobal("MobMapUberQuestQuestLogButton"..i);
			if(frame:IsVisible()) then
				local questobj=frame:GetText();
				if(questobj) then
					if(button==nil) then
						button=CreateFrame("Frame","MobMapUberQuestQuestLogButton"..i,UberQuest_Details_ScrollChild,"MobMapQuestButtonFrameTemplate");
						button:ClearAllPoints();
						button:SetFrameStrata("MEDIUM");
						button:SetPoint("LEFT","UberQuest_Details_ScrollChild_Objective"..i,"RIGHT",-16,0);
						button:SetAlpha(0.6);
					end
					button.questobj=questobj;
					button:Show();
				end
			else
				if(button~=nil) then button:Hide(); end
			end
		end
	end
end

function MobMapQuestLogButtons_HideAll()
	local i;
	frame=getglobal("MobMapQuestLogTitleButton");
	if(frame~=nil) then frame:Hide(); end
	for i=1,10,1 do
		frame=getglobal("MobMapQuestLogButton"..i);
		if(frame~=nil) then frame:Hide(); end
	end
	frame=getglobal("MobMapELQ3QuestLogTitleButton");
	if(frame~=nil) then frame:Hide(); end
	for i=1,10,1 do
		frame=getglobal("MobMapELQ3QuestLogButton"..i);
		if(frame~=nil) then frame:Hide(); end
	end
end

-- item name selection frame

function MobMap_ShowItemNameSelectionFrame(startihid)
	MobMap_LoadDatabase(MOBMAP_ITEMNAME_HELPER_DATABASE);
	if(startihid~=nil) then
		mobmap_itemnameselection_selecteditem=startihid;
	else
		mobmap_itemnameselection_selecteditem=nil;
	end
	MobMap_RefreshItemNameSelectionFrame();
	MobMapItemNameSelectionFrame:Show();
end

mobmap_itemnameselectionlist=nil;

function MobMap_RefreshItemNameSelectionFrame()
	local filtername=MobMapItemNameSelectionFrameNameFilter:GetText();
	mobmap_itemnameselectionlist=MobMap_GetItemNameList(filtername);
	FauxScrollFrame_SetOffset(MobMapItemNameSelectionFrameScrollFrame, 0);
	MobMap_UpdateItemNameSelectionFrame();
end

function MobMap_UpdateItemNameSelectionFrame()
	local itemnamecount=table.getn(mobmap_itemnameselectionlist);
	local offset=FauxScrollFrame_GetOffset(MobMapItemNameSelectionFrameScrollFrame);
	MobMapItemNameSelectionFrameItemHighlightFrame:Hide();

	for i=1,9,1 do
		local itemindex=i+offset;
		if(itemindex>itemnamecount) then
			MobMap_UpdateItemNameEntry(i, nil);
		else
			MobMap_UpdateItemNameEntry(i, mobmap_itemnameselectionlist[itemindex]);
		end
	end

	FauxScrollFrame_Update(MobMapItemNameSelectionFrameScrollFrame, itemnamecount, 9, 22);
end

function MobMap_UpdateItemNameEntry(pos, ihid)
	local frame=getglobal("MobMapItemSelection"..pos);
	local frame_button=getglobal("MobMapItemSelection"..pos.."ItemName");
	local frame_text=getglobal("MobMapItemSelection"..pos.."ItemNameText");
	if(ihid==nil) then
		frame:Hide();
	else
		local itemname=MobMap_GetItemNameByIHID(ihid);
		local itemid, quality = MobMap_GetItemDataByIHID(ihid);
		if(itemname~=nil and itemid~=nil) then
			local r, g, b = GetItemQualityColor(quality);
			frame_text:SetText(itemname);
			frame_text:SetTextColor(r,g,b);
			frame.itemid=itemid;
			frame.ihid=ihid;
			frame:Show();
			if(ihid==mobmap_itemnameselection_selecteditem) then
				MobMapItemNameSelectionFrameItemHighlightFrame:Show();
				MobMapItemNameSelectionFrameItemHighlightFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 2);
				MobMapItemNameSelectionFrameItemHighlightFrame:SetAlpha(0.5);
				MobMapItemHighlight:SetVertexColor(1.0, 1.0, 1.0);
			end
		end
	end
end

mobmap_itemnameselection_selecteditem=nil;
mobmap_itemnameselection_selectionfunc=nil;

function MobMap_SelectItem()
	MobMapItemNameSelectionFrameItemHighlightFrame:Show();
	MobMapItemNameSelectionFrameItemHighlightFrame:SetPoint("TOPLEFT", this, "TOPLEFT", 0, 2);
	MobMapItemNameSelectionFrameItemHighlightFrame:SetAlpha(0.5);
	MobMapItemHighlight:SetVertexColor(1.0, 1.0, 1.0);
	mobmap_itemnameselection_selecteditem=this:GetParent().ihid;
end

function MobMap_CallSelectionFunc()
	if(mobmap_itemnameselection_selectionfunc~=nil and mobmap_itemnameselection_selecteditem~=nil) then
		mobmap_itemnameselection_selectionfunc(mobmap_itemnameselection_selecteditem);
	end
end

function MobMap_SetSelectionFunc(selectionfunc)
	mobmap_itemnameselection_selectionfunc=selectionfunc;
end

-- about frame

function MobMap_AboutFrame_OnShow()
	local buildtimestring;
	if(GetLocale()=="deDE") then
		buildtimestring=date("%d.%m.%Y um %H:%M Uhr", MobMap_GetDBBuildTime());
	else
		buildtimestring=date("%m/%d/%y at %I:%M %p", MobMap_GetDBBuildTime());
	end
	MobMapAboutFrameDBInfoBuildTime:SetText(MOBMAP_DATABASE_INFO_1..buildtimestring);
	local language;
	if(MobMap_GetDBLanguage()=="ger") then language=MOBMAP_DATABASE_LANGUAGE_GERMAN; end
	if(MobMap_GetDBLanguage()=="eng") then language=MOBMAP_DATABASE_LANGUAGE_ENGLISH; end
	MobMapAboutFrameDBInfoLanguage:SetText(MOBMAP_DATABASE_INFO_2..language);
	local mobcountstring=MobMap_GetMobCount();
	MobMapAboutFrameDBInfoMobCount:SetText(MOBMAP_DATABASE_INFO_3..mobcountstring);
	local questcountstring;
	if(MobMap_GetQuestCount()==-1) then
		questcountstring=MOBMAP_DATABASE_INFO_NOT_INSTALLED;
	else
		questcountstring=MobMap_GetQuestCount();
	end
	MobMapAboutFrameDBInfoQuestCount:SetText(MOBMAP_DATABASE_INFO_4..questcountstring);
	local merchantcountstring;
	if(MobMap_GetMerchantCount()==-1) then
		merchantcountstring=MOBMAP_DATABASE_INFO_NOT_INSTALLED;
	else
		merchantcountstring=MobMap_GetMerchantCount();
	end
	MobMapAboutFrameDBInfoMerchantCount:SetText(MOBMAP_DATABASE_INFO_5..merchantcountstring);
	local recipecountstring;
	if(MobMap_GetRecipeCount()==-1) then
		recipecountstring=MOBMAP_DATABASE_INFO_NOT_INSTALLED;
	else
		recipecountstring=MobMap_GetRecipeCount();
	end
	MobMapAboutFrameDBInfoRecipeCount:SetText(MOBMAP_DATABASE_INFO_6..recipecountstring);

end

-- export frame

function MobMap_ExportData()
	local continueExport=true;
	local complete_datastring="<MobMapData language=\""..GetLocale().."\" version=\"41\">";
	local mobmap_export_datastring="";
	local targetname, zones, zone, positions, position, value;
	for targetname,zones in pairs(mobmap_positions) do
		if(zones["min"]~=nil and zones["max"]~=nil) then
			local datastring_to_add="";
			datastring_to_add=datastring_to_add.."<Mob name=\""..MobMap_XMLArgumentEscape(targetname).."\" min=\""..zones["min"].."\" max=\""..zones["max"].."\"";	
			datastring_to_add=datastring_to_add..">";
			for zone,positions in pairs(zones) do
				if(zone~="min" and zone~="max" and zone~="sub") then
					datastring_to_add=datastring_to_add.."<Zone name=\""..MobMap_XMLArgumentEscape(zone).."\">";
					for position,value in pairs(positions) do
						datastring_to_add=datastring_to_add..position..",";
					end
					datastring_to_add=datastring_to_add.."</Zone>";
				end
			end
			datastring_to_add=datastring_to_add.."</Mob>";

			if(string.len(complete_datastring)+string.len(mobmap_export_datastring)+string.len(datastring_to_add) > MOBMAP_EXPORT_MAXSIZE) then
				continueExport=false;
				break;
			else
				continueExport=true;
				mobmap_export_datastring=mobmap_export_datastring..datastring_to_add;
			end
		end
	end

	complete_datastring=complete_datastring..mobmap_export_datastring;
	mobmap_export_datastring="";

	if(continueExport) then
		local questname, questdata, rewardid;
		for questname, questdata in pairs(mobmap_quests) do
			local datastring_to_add="";
			local realquestname=string.match(questname,"(.*)|.*");
			if(realquestname==nil) then realquestname=questname; end
			if(questdata["group"]==nil) then questdata["group"]="solo"; end
			if(questdata["faction"]==nil) then questdata["faction"]="Alliance"; end
			datastring_to_add=datastring_to_add.."<Quest name=\""..MobMap_XMLArgumentEscape(realquestname).."\" level=\""..questdata["level"].."\" npc=\""..MobMap_XMLArgumentEscape(questdata["npc"]).."\" zone=\""..MobMap_XMLArgumentEscape(questdata["zone"]).."\" obj=\""..MobMap_QuestTextEscape(MobMap_XMLArgumentEscape(questdata["obj"])).."\" group=\""..questdata["group"].."\" faction=\""..questdata["faction"].."\">";
			if(questdata["choice"]~=nil) then
				for _,rewardid in pairs(questdata["choice"]) do
					datastring_to_add=datastring_to_add.."<Choice id=\""..rewardid.."\"/>";
				end
			end
			if(questdata["always"]~=nil) then
				for _,rewardid in pairs(questdata["always"]) do
					datastring_to_add=datastring_to_add.."<Always id=\""..rewardid.."\"/>";
				end
			end
			datastring_to_add=datastring_to_add.."</Quest>";

			if(string.len(complete_datastring)+string.len(mobmap_export_datastring)+string.len(datastring_to_add) > MOBMAP_EXPORT_MAXSIZE) then
				continueExport=false;
				break;
			else
				continueExport=true;
				mobmap_export_datastring=mobmap_export_datastring..datastring_to_add;
			end
		end

		complete_datastring=complete_datastring..mobmap_export_datastring;
		mobmap_export_datastring="";
	end

	if(continueExport) then
		local merchant, merchantdata;
		for merchant, merchantdata in pairs(mobmap_merchantstock) do
			local datastring_to_add="";
			datastring_to_add=datastring_to_add.."<Merchant name=\""..MobMap_XMLArgumentEscape(merchant).."\">";
			local item, itemdata;
			for item=1, 100, 1 do
				local itemdata=merchantdata[tostring(item)];
				if(itemdata) then
					datastring_to_add=datastring_to_add.."<MItem id=\""..itemdata["id"].."\" qty=\""..itemdata["quantity"].."\" ltd=\""..itemdata["limited"].."\" cost=\""..itemdata["price"];
					if(itemdata["arenaprice"]) then
						datastring_to_add=datastring_to_add.."\" arena=\""..itemdata["arenaprice"];
					end
					if(itemdata["honorprice"]) then
						datastring_to_add=datastring_to_add.."\" honor=\""..itemdata["honorprice"];
					end
					local token;
					for token=1,5,1 do
						if(itemdata["token"..token]) then
							datastring_to_add=datastring_to_add.."\" token"..token.."=\""..MobMap_XMLArgumentEscape(itemdata["token"..token]);
						end
					end
					datastring_to_add=datastring_to_add.."\"/>";
				end
			end
			datastring_to_add=datastring_to_add.."</Merchant>";

			if(string.len(complete_datastring)+string.len(mobmap_export_datastring)+string.len(datastring_to_add) > MOBMAP_EXPORT_MAXSIZE) then
				continueExport=false;
				break;
			else
				continueExport=true;
				mobmap_export_datastring=mobmap_export_datastring..datastring_to_add;
			end
		end

		complete_datastring=complete_datastring..mobmap_export_datastring;
		mobmap_export_datastring="";
	end

	if(continueExport) then
		local skillName, skillData;
		for skillName, skillData in pairs(mobmap_tradeskills) do
			local datastring_to_add="";
			datastring_to_add=datastring_to_add.."<Recipe name=\""..MobMap_XMLArgumentEscape(skillName).."\" skill=\""..MobMap_XMLArgumentEscape(skillData["category"]).."\" level=\""..skillData["level"].."\"";
			if(skillData["itemid"]~=nil) then datastring_to_add=datastring_to_add.." itemid=\""..skillData["itemid"].."\""; end
			if(skillData["enchantid"]~=nil) then datastring_to_add=datastring_to_add.." enchid=\""..skillData["enchantid"].."\""; end
			if(skillData["minmade"]~=nil) then datastring_to_add=datastring_to_add.." mincnt=\""..skillData["minmade"].."\""; end
			if(skillData["maxmade"]~=nil) then datastring_to_add=datastring_to_add.." maxcnt=\""..skillData["maxmade"].."\""; end
			datastring_to_add=datastring_to_add..">";
			local reagent;
			for reagent=1,10,1 do
				if(skillData["reagent"..reagent]~=nil) then
					datastring_to_add=datastring_to_add.."<Reagent id=\""..skillData["reagent"..reagent].itemid.."\" count=\""..skillData["reagent"..reagent].count.."\"/>";
				end
			end
			datastring_to_add=datastring_to_add.."</Recipe>";

			if(string.len(complete_datastring)+string.len(mobmap_export_datastring)+string.len(datastring_to_add) > MOBMAP_EXPORT_MAXSIZE) then
				continueExport=false;
				break;
			else
				continueExport=true;
				mobmap_export_datastring=mobmap_export_datastring..datastring_to_add;
			end
		end

		complete_datastring=complete_datastring..mobmap_export_datastring;
		mobmap_export_datastring="";
	end

	if(continueExport) then
		local lootName, lootData;
		for lootName, lootData in pairs(mobmap_loot) do
			local datastring_to_add="<Loot name=\""..MobMap_XMLArgumentEscape(lootName).."\">"..lootData.."</Loot>";

			if(string.len(complete_datastring)+string.len(mobmap_export_datastring)+string.len(datastring_to_add) > MOBMAP_EXPORT_MAXSIZE) then
				continueExport=false;
				break;
			else
				continueExport=true;
				mobmap_export_datastring=mobmap_export_datastring..datastring_to_add;
			end
		end
	end

	mobmap_export_datastring=mobmap_export_datastring.."</MobMapData>";
	complete_datastring=complete_datastring..mobmap_export_datastring;
	mobmap_export_datastring=MobMap_Base64(complete_datastring);
	MobMapExportFrameData.data=mobmap_export_datastring;
	MobMapExportFrameData:SetText(mobmap_export_datastring);
	MobMapExportFrame:Show();
end

function MobMap_DeletePositionData()
	mobmap_positions={};
	mobmap_quests={};
	mobmap_merchantstock={};
	mobmap_tradeskills={};
	mobmap_loot={};
end

function MobMapExportDelayFrame_OnUpdate()
	if(mobmap_export_status==3) then MobMap_ExportData(); end
end

-- error frame

function MobMap_ErrorMessage(msg)
	MobMapErrorMessageFrameText:SetText(msg);
	MobMapErrorMessageFrame:Show();
end

-- options frame

function MobMap_OptionsFrame_OnShow()
	MobMapOptionsFrameDotOpacitySlider:SetValue(mobmap_dot_transparency*100);
	MobMapOptionsFrameAlternateButtonPosCheckButton:SetChecked(mobmap_button_position==2);
	MobMapOptionsFrameAlternateButtonPos2CheckButton:SetChecked(mobmap_button_position==3);
	MobMapOptionsFrameHideWorldmapButtonCheckButton:SetChecked(mobmap_button_position==0);
	MobMapOptionsFrameDatabaseLoadingInfoCheckButton:SetChecked(mobmap_display_database_loading_info);
	MobMapOptionsFrameWorldMapTooltipCheckButton:SetChecked(mobmap_show_world_map_tooltips);
	if(mobmap_dot_scaling_behavior==0) then
		MobMapOptionsFrameDotScaling1CheckButton:SetChecked(true);
	else
		MobMapOptionsFrameDotScaling1CheckButton:SetChecked(false);
	end
	if(mobmap_dot_scaling_behavior==1) then
		MobMapOptionsFrameDotScaling2CheckButton:SetChecked(true);
	else
		MobMapOptionsFrameDotScaling2CheckButton:SetChecked(false);
	end
	if(mobmap_dot_scaling_behavior==2) then
		MobMapOptionsFrameDotScaling3CheckButton:SetChecked(true);
	else
		MobMapOptionsFrameDotScaling3CheckButton:SetChecked(false);
	end
end

function MobMap_RescalePositionDots()
	if(mobmap_displayed_dot_count==nil) then return; end
	if(mobmap_dot_scaling_behavior==0) then
		if(mobmap_displayed_dot_count>3) then
			mobmap_dot_scale=1.0;
		else
			mobmap_dot_scale=2.0;
		end
	elseif(mobmap_dot_scaling_behavior==1) then
		mobmap_dot_scale=1.0;
	elseif(mobmap_dot_scaling_behavior==2) then
		mobmap_dot_scale=2.0;
	end
end

-- general utilities

function MobMap_Command(cmd)
	local parts={};
	local partcount=0;
	for w in string.gmatch(cmd, "%S+") do
    		parts[partcount]=w;
	    	partcount=partcount+1;
	end
	if(partcount==0) then
		MobMap_DisplayMessage(MOBMAP_COMMANDS1);
		MobMap_DisplayMessage(MOBMAP_COMMANDS2);
		MobMap_DisplayMessage(MOBMAP_COMMANDS3);
    		return;
	end
	if(partcount==1) then
		if(parts[0]=="show") then
			MobMap_ShowMobMapFrame();
		end
		if(parts[0]=="clear") then
			MobMap_DeletePositionData();
			MobMap_DisplayMessage(MOBMAP_CLEARED);
		end
	end
end

function MobMap_Base64(str)
	local encodedString="";
	local partcount;
	for partcount=0,ceil(string.len(str)/300)-1,1 do
		local substringlength=string.len(str)-(partcount*300);
		if(substringlength>300) then substringlength=300; end
		local substring=string.sub(str,partcount*300+1,partcount*300+substringlength);
		local encodedSubstring=base64_encode(substring);
		encodedString=encodedString..encodedSubstring;
	end
	return encodedString;
end

function MobMap_GetPlayerCoordinates()
	if(not WorldMapFrame:IsVisible()) then
		SetMapToCurrentZone();
		local x, y = GetPlayerMapPosition("player");
		local zonename = GetRealZoneText();
		if(x>0 and y>0) then
			x=math.floor(x*100+0.5);
			y=math.floor(y*100+0.5);
		else
			x=-1;
			y=-1;
		end
		return x,y,zonename;
	end
	return nil;
end

function MobMap_GetMoneyFromMoneyString(str)
	local gold, silver, copper;
	gold=string.match(str,"(%d+) "..GOLD);
	if(gold==nil) then gold=0; end
	silver=string.match(str,"(%d+) "..SILVER);
	if(silver==nil) then silver=0; end
	copper=string.match(str,"(%d+) "..COPPER);
	if(copper==nil) then copper=0; end	
	return gold*10000+silver*100+copper;
end

function MobMap_GetItemIDFromItemLink(link)
	local found, _, itemString = string.find(link, "^|%x+|H(.+)|h%[.+%]")
	local found2, _, itemId, enchantId, jewelId1, jewelId2, jewelId3, jewelId4, suffixId, uniqueId = string.find(itemString, "^item:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%-?%d+):(%-?%d+)")
	return itemId, suffixId, uniqueId;
end

function MobMap_GetItemIDFromItemString(str)
	local found, _, itemId, enchantId, jewelId1, jewelId2, jewelId3, jewelId4, suffixId, uniqueId = string.find(str, "^item:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%-?%d+):(%-?%d+)")
	return itemId;
end

function MobMap_GetEnchantIDFromEnchantLink(link)
	local found, _, itemString = string.find(link, "^|%x+|H(.+)|h%[.+%]")
	local found2, _, enchantId = string.find(itemString, "^enchant:(%d+)")
	return enchantId;
end

function MobMap_ConstructEnchantLink(enchantid, enchantname)
	local enchantLink="|Cffffffff|Henchant:"..enchantid.."|h["..enchantname.."]|h|r";
	return enchantLink;
end

function MobMap_ConstructItemString(itemid)
	local itemString="item:"..itemid..":0:0:0:0:0:0:0";
	return itemString;
end

function MobMap_DisplayMessage(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg);
end

function MobMap_XMLArgumentEscape(str)
	str=string.gsub(str, "&", "&amp;");
	str=string.gsub(str, "\"", "&quot;");
	str=string.gsub(str, "'", "&apos;");
	str=string.gsub(str, "<", "&lt;");
	str=string.gsub(str, ">", "&gt;");
	str=string.gsub(str, "ü", "|uuml|");
	str=string.gsub(str, "ä", "|auml|");
	str=string.gsub(str, "ö", "|ouml|");
	str=string.gsub(str, "Ü", "|Uuml|");
	str=string.gsub(str, "Ä", "|Auml|");
	str=string.gsub(str, "Ö", "|Ouml|");
	str=string.gsub(str, "ß", "|szlig|");
	str=string.gsub(str, "\n", "$B");
	str=string.gsub(str, "\r", "");
	return str;
end

function MobMap_QuestTextEscape(str)
	str=string.gsub(str, "\r\n", "\n");
	str=string.gsub(str, "\n", "\\n");
	return str;
end

function MobMap_PatternEscape(str)
	if(str==nil) then return nil; end
	str=string.gsub(str,"%^","%%%^");
	str=string.gsub(str,"%(","%%%(");
	str=string.gsub(str,"%)","%%%)");
	str=string.gsub(str,"%%","%%%%");
	str=string.gsub(str,"%.","%%%.");
	str=string.gsub(str,"%[","%%%[");
	str=string.gsub(str,"%]","%%%]");
	str=string.gsub(str,"%*","%%%*");
	str=string.gsub(str,"%+","%%%+");
	str=string.gsub(str,"%-","%%%-");
	str=string.gsub(str,"%?","%%%?");
	return str;	
end

function MobMap_LoadDatabase(dbtype)
	if(dbtype==MOBMAP_POSITION_DATABASE) then
		if(MobMap_LoadDatabase(MOBMAP_MOBNAME_DATABASE)==false) then return false; end
	elseif(dbtype==MOBMAP_QUEST_DATABASE) then
		if(MobMap_LoadDatabase(MOBMAP_MOBNAME_DATABASE)==false) then return false; end
	elseif(dbtype==MOBMAP_MERCHANT_DATABASE) then
		if(MobMap_LoadDatabase(MOBMAP_MOBNAME_DATABASE)==false) then return false; end
	elseif(dbtype==MOBMAP_RECIPE_DATABASE) then
		if(MobMap_LoadDatabase(MOBMAP_ITEMNAME_HELPER_DATABASE)==false) then return false; end
	end
	if(IsAddOnLoaded("MobMapDatabaseStub"..dbtype)==nil) then
		local startTime=GetTime();
		if(mobmap_display_database_loading_info==true) then collectgarbage(); end
		local startRAM=gcinfo();
		local gcTime=floor((GetTime()-startTime)*1000)/1000;
		startTime=GetTime();
		local loaded,reason=LoadAddOn("MobMapDatabaseStub"..dbtype);
		local whatWasLoaded="position";
		if(dbtype==MOBMAP_QUEST_DATABASE) then whatWasLoaded="quest"; end
		if(dbtype==MOBMAP_MERCHANT_DATABASE) then whatWasLoaded="merchant"; end
		if(dbtype==MOBMAP_RECIPE_DATABASE) then whatWasLoaded="recipe"; end
		if(dbtype==MOBMAP_ITEMNAME_HELPER_DATABASE) then whatWasLoaded="item name helper"; end
		if(dbtype==MOBMAP_MOBNAME_DATABASE) then whatWasLoaded="mob name"; end
		if(not loaded) then
			MobMap_DisplayMessage("The MobMap "..whatWasLoaded.." database could not be dynamically loaded! You might try to uninstall and reinstall MobMap to fix this problem.");
			return false;
		else		
			local loadTime=floor((GetTime()-startTime)*1000)/1000;
			startTime=GetTime();
			collectgarbage();
			local usedRAM=gcinfo()-startRAM;
			local gcTime=gcTime+floor((GetTime()-startTime)*1000)/1000;
			if(mobmap_display_database_loading_info==true) then
				MobMap_DisplayMessage("The MobMap "..whatWasLoaded.." database just got dynamically loaded in "..loadTime.." seconds (+ "..gcTime.." seconds GC time) and occupies "..usedRAM.." kbytes of addon memory now.");
			end
			return true;
		end
	end
end

function MobMap_Mask(number1, number2)
	if(MOBMAP_ISONAMAC) then
		if(number1<0) then 
			return 0; 
		else 
			return floor(number1%number2); 
		end
	else
		return bit.band(number1,number2-1);
	end
end

function MobMap_RGBToHex(r, g, b)
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end

-- this is actually a copy of a Blizzard function to get the quest difficulty colors

MobMap_QuestDifficultyColor = { };
MobMap_QuestDifficultyColor["impossible"] = { r = 1.00, g = 0.10, b = 0.10 };
MobMap_QuestDifficultyColor["verydifficult"] = { r = 1.00, g = 0.50, b = 0.25 };
MobMap_QuestDifficultyColor["difficult"] = { r = 1.00, g = 1.00, b = 0.00 };
MobMap_QuestDifficultyColor["standard"] = { r = 0.25, g = 0.75, b = 0.25 };
MobMap_QuestDifficultyColor["trivial"]	= { r = 0.50, g = 0.50, b = 0.50 };
MobMap_QuestDifficultyColor["header"]	= { r = 0.7, g = 0.7, b = 0.7 };

function MobMap_GetQuestDifficultyColor(level)
	local levelDiff = level - UnitLevel("player");
	if ( levelDiff >= 5 ) then
		color = MobMap_QuestDifficultyColor["impossible"];
	elseif ( levelDiff >= 3 ) then
		color = MobMap_QuestDifficultyColor["verydifficult"];
	elseif ( levelDiff >= -2 ) then
		color = MobMap_QuestDifficultyColor["difficult"];
	elseif ( -levelDiff <= GetQuestGreenRange() ) then
		color = MobMap_QuestDifficultyColor["standard"];
	else
		color = MobMap_QuestDifficultyColor["trivial"];
	end
	return color;	
end