-- position database functions

function MobMap_GetMobPositions(mobid, zoneid)
	if(mobmap_mobs[mobid]==nil) then return nil; end
	local mobpointer=MobMap_GetMobPointer(mobid);
	local minlevel, maxlevel, zonecount = MobMap_GetMobDetails(mobpointer);
	local i;
	for i=1, zonecount, 1 do
		local zonecode, length, pointer = MobMap_GetPointerToPositionData(mobpointer+i);
		if(zonecode==zoneid) then
			return MobMap_GetCoordPairsFromDB(pointer,length);
		end
	end
	return nil;
end

function MobMap_GetCoordPairsFromDB(position, length)
	local i;
	local coords={};
	local xo,yo;
	for i=0, length-1, 1 do
		local xc,yc=MobMap_GetCoordFromDB(position+i)
		if(i%2==0) then
			xo=xc;
			yo=yc;
		else
			table.insert(coords, {x1=xo, x2=xc, y=yo});
		end
	end
	return coords;
end

function MobMap_GetCoordFromDB(position)
	local bucket=floor(position/4)+1;
	local data=mobmap_data[bucket];
	if(data==nil) then return nil; end
	local x;
	local y=0;
	x=floor(data/(mobmap_shiftconst[(position%4)*2])+0.5);
	if(position%2==0) then y=floor(data/(mobmap_shiftconst[(position%4)*2+1])+0.5); end
	x=bit.band(x,0xFF);
	y=bit.band(y,0xFF);
	return x,y;
end

-- user interface functions

mobmap_currentlist = {};
mobmap_zonelist = {};
mobmap_currentlyshown = nil;
mobmap_multidisplay = nil;

function MobMap_UpdateMobMapFrame()
	for i=1,15,1 do
		local frame=getglobal("MobMapMob"..i);
		if(mobmap_currentlist[i]~=nil) then
			frame:Show();
			frame:SetText(mobmap_currentlist[i].name);
			frame.subtitle=mobmap_currentlist[i].sub;
			frame.mobid=mobmap_currentlist[i].mobid;
			frame:Enable();
		else
			frame:Hide();
		end
	end
	if(mobmap_currentlist[16]~=nil) then
		MobMapMob15:SetText(MOBMAP_MORE_MOBS_AVAILABLE);
		MobMapMob15:Disable();
	end
	if(mobmap_currentlist[1]==nil and string.sub(MobMapMobSearchFrameSearchBox:GetText(),1,3)~="id:") then
		MobMapMob1:SetText(MOBMAP_NO_MOBS_FOUND);
		MobMapMob1:Disable();
		MobMapMob1:Show();
		MobMap_UpdateZoneList();
	end
end

function MobMap_UpdateFilter(text, subtitle)
	mobmap_currentlist = {};
	local filtertext;
	local exactmatch;
	if(string.sub(text,1,1)=="\"" and string.sub(text,-1)=="\"") then
		exactmatch=true;
		filtertext=string.sub(text,2,string.len(text)-1);
	else
		filtertext=text;
		exactmatch=false;
	end
	if(string.sub(text,1,3)=="id:") then
		mobmap_currentlist={};
		mobmap_multidisplay={};
		text=string.sub(text,4);
		for w in string.gmatch(text, "%d*") do
			if(tonumber(w)~=nil) then
				table.insert(mobmap_multidisplay,{id=tonumber(w)});
			end
		end
		if(table.getn(mobmap_multidisplay)==0) then 
			mobmap_multidisplay=nil; 
		else
			MobMap_UpdatePositions();
			MobMap_SwitchMapAndDisplay();
		end
	else
		if(mobmap_multidisplay~=nil) then
			mobmap_multidisplay=nil;
			MobMap_HideAllDots();
		end
		filtertext=MobMap_PatternEscape(filtertext);
		subtitle=MobMap_PatternEscape(subtitle);
		for k,v in pairs(mobmap_mobs) do
			local part1,part2=MobMap_GetMobNameAndSubtitle(v);
			if((exactmatch==false and string.find(string.lower(part1),".-"..string.lower(filtertext)..".-")~=nil) or (exactmatch==true and string.lower(part1)==string.lower(filtertext))) then
				if((part2==nil and subtitle=="") or (part2~=nil and string.find(string.lower(part2),".-"..string.lower(subtitle)..".-")~=nil)) then
					table.insert(mobmap_currentlist,{name=part1, sub=part2, mobid=k});
					if(table.getn(mobmap_currentlist)>15) then break; end
				end
			end
		end
	end
	MobMap_UnsetMob();
	MobMap_UpdateMobMapFrame();
	if(table.getn(mobmap_currentlist)==1) then
		MobMapButton_ProcessClick("MobMapMob1");
	else
		mobmap_zonelist={};
		MobMap_UpdateZoneList();
	end
end

function MobMapButton_OnClick()
	MobMapButton_ProcessClick(this:GetName());
end

function MobMapButton_ProcessClick(button)
	local frame=getglobal(button);
	MobMapMobSearchFrameMobHighlightFrame:Show();
	MobMapMobSearchFrameMobHighlightFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0);
	MobMapMobSearchFrameMobHighlightFrame:SetAlpha(0.5);
	MobMapHighlight:SetVertexColor(1.0, 1.0, 1.0);
	MobMapMobSearchFrameSelectionDetails:Show();
	MobMapMobSearchFrameSelectionDetailsName:SetText(frame:GetText());
	if(frame.subtitle~=nil) then
		MobMapMobSearchFrameSelectionDetailsSubtitle:SetText(frame.subtitle);
		MobMapMobSearchFrameSelectionDetails.mobfullname=frame:GetText().."|"..frame.subtitle;
		MobMapMobSearchFrameSelectionDetails.mobid=frame.mobid;
	else
		MobMapMobSearchFrameSelectionDetailsSubtitle:SetText("");
		MobMapMobSearchFrameSelectionDetails.mobfullname=frame:GetText();
		MobMapMobSearchFrameSelectionDetails.mobid=frame.mobid;
	end
	local pointer=MobMap_GetMobPointer(frame.mobid);
	if(pointer==nil) then return; end
	local minlevel, maxlevel, zonecount = MobMap_GetMobDetails(pointer);
	if(minlevel~=maxlevel) then
		MobMapMobSearchFrameSelectionDetailsLevel:SetText(MOBMAP_LEVEL..minlevel.." - "..maxlevel);
	else
		if(minlevel==0) then
			MobMapMobSearchFrameSelectionDetailsLevel:SetText(MOBMAP_BOSS_LEVEL);
		else
			MobMapMobSearchFrameSelectionDetailsLevel:SetText(MOBMAP_LEVEL..minlevel);
		end
	end
	mobmap_zonelist={};
	local zonelist=MobMap_GetMobZones(pointer);
	for k,v in pairs(zonelist) do
		table.insert(mobmap_zonelist,{name=MobMap_GetZoneName(v), id=v});
	end
	MobMap_UnsetZone();
	MobMap_UpdateZoneList();
end

function MobMap_UpdateZoneList()
	local zones=table.getn(mobmap_zonelist);
	local offset=FauxScrollFrame_GetOffset(MobMapMobSearchFrameZoneListScrollFrame);

	for i=1,13,1 do
		local zoneindex=i+offset;
		local frame=getglobal("MobMapZone"..i);
		if(zoneindex>zones) then
			frame:Hide();
		else
			frame:SetText(mobmap_zonelist[zoneindex].name);
			frame.id=mobmap_zonelist[zoneindex].id;
			frame:Show();
		end
	end

	FauxScrollFrame_Update(MobMapMobSearchFrameZoneListScrollFrame, zones, 13, 22);

	if(zones==1) then
		MobMapZoneButton_ProcessClick("MobMapZone1");
		if(mobmap_questsearch) then HideUIPanel(MobMapFrame); end
	else
		if(not MobMapFrame:IsVisible()) then ShowUIPanel(MobMapFrame); end
		if(MobMapMobSearchFrame~=nil and not MobMapMobSearchFrame:IsVisible()) then MobMap_ShowPanel("MobMapMobSearchFrame"); end
	end
	mobmap_questsearch=false;
end

function MobMapZoneButton_OnClick()
	MobMapZoneButton_ProcessClick(this:GetName());
end

function MobMapZoneButton_ProcessClick(button)
	local frame=getglobal(button);
	MobMapZoneHighlightFrame:Show();
	MobMapZoneHighlightFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0);
	MobMapZoneHighlightFrame:SetAlpha(0.5);
	MobMapZoneHighlight:SetVertexColor(1.0, 1.0, 1.0);
	mobmap_multidisplay=nil;
	mobmap_currentlyshown={zonename=frame:GetText(), zoneid=frame.id, mobname=MobMapMobSearchFrameSelectionDetailsName:GetText(), mobfullname=MobMapMobSearchFrameSelectionDetails.mobfullname, mobid=MobMapMobSearchFrameSelectionDetails.mobid};
	mobmap_enabled=true;
	MobMapCheckButton:SetChecked(true);
	MobMap_SwitchMapAndDisplay();
end

function MobMap_UnsetMob()
	MobMapMobSearchFrameMobHighlightFrame:Hide();
	MobMapMobSearchFrameSelectionDetails:Hide();
	MobMap_UnsetZone();
end

function MobMap_UnsetZone()
	mobmap_currentlyshown=nil;
	MobMapZoneHighlightFrame:Hide();
end

function MobMap_UpdatePositions()
	if(mobmap_currentlyshown==nil) then 
		if(mobmap_multidisplay~=nil) then
			local i,k;
			local zonestats={};
			for i=1,table.getn(mobmap_multidisplay),1 do
				local zonetable=MobMap_GetMobZonesByMobID(mobmap_multidisplay[i].id);
				if(zonetable~=nil) then
					mobmap_multidisplay[i].zones={};
					for k=1,table.getn(zonetable),1 do
						local positionData=MobMap_GetMobPositions(mobmap_multidisplay[i].id, zonetable[k]);
						table.insert(mobmap_multidisplay[i].zones,{zoneid=zonetable[k], posdata=positionData});
						if(zonestats[zonetable[k]]==nil) then
							zonestats[zonetable[k]]=1;
						else
							zonestats[zonetable[k]]=zonestats[zonetable[k]]+1;
						end
					end
					local maxzone=0;
					local maxvalue=0;
					local key, value;
					for key, value in pairs(zonestats) do
						if(value>maxvalue) then maxzone=key; end
					end
					if(maxzone~=0) then
						if(not WorldMapFrame:IsVisible()) then WorldMapFrame:Show(); end
						MobMap_SetMapToZone(MobMap_GetZoneName(maxzone));
					end
				end
			end
		end
		return;
	end
	local pos=MobMap_GetMobPositions(mobmap_currentlyshown.mobid, mobmap_currentlyshown.zoneid);
	mobmap_currentlyshown.posdata=pos;
end

mobmap_astrolabe_version = "Astrolabe-0.3";
mobmap_astrolabe_library = nil;

function MobMap_Display()
	if(WorldMapFrame:IsVisible()==false) then return; end
	if(mobmap_currentlyshown==nil) then 
		if(mobmap_multidisplay==nil) then
			MobMapButtonFrameCurrentMob:SetText("");
			MobMap_HideAllDots();
			return;
		else
			MobMap_SwitchMapAndDisplay();
			return;
		end
	end
	MobMapButtonFrameCurrentMob:SetText(mobmap_currentlyshown.mobname);
	if(MobMap_GetCurrentMapZoneName()~=mobmap_currentlyshown.zonename) then 
		MobMap_HideAllDots();
		return;
	end
	if(mobmap_enabled) then
		MobMap_SwitchMapAndDisplay();
	else
		MobMap_HideAllDots();
	end
end

function MobMap_SwitchMapAndDisplay()
	mobmap_displayed_dot_count=0;
	if(mobmap_currentlyshown==nil) then 
		if(mobmap_multidisplay==nil) then
			MobMapButtonFrameCurrentMob:SetText("");
			return;
		else
			local showed=false;
			if(not WorldMapFrame:IsVisible()) then 
				ShowUIPanel(WorldMapFrame); 
				MobMapFrame:Hide();
				MobMapFrame:Show();
				showed=true;
			end
			local i,k;
			MobMap_HideAllDots();
			for i=1,table.getn(mobmap_multidisplay),1 do
				if(mobmap_multidisplay[i].zones~=nil) then
					for k=1,table.getn(mobmap_multidisplay[i].zones) do
						if(mobmap_multidisplay[i].zones[k].posdata~=nil and MobMap_GetZoneName(mobmap_multidisplay[i].zones[k].zoneid)==MobMap_GetCurrentMapZoneName()) then
							MobMap_DisplayPositionData(mobmap_multidisplay[i].zones[k].posdata, mobmap_multidisplay[i].id);
						end
					end
				end
			end
			return;
		end
	end
	local showed=false;
	if(not WorldMapFrame:IsVisible()) then 
		ShowUIPanel(WorldMapFrame); 
		MobMapFrame:Hide();
		MobMapFrame:Show();
		showed=true;
	end
	if(MobMap_SetMapToZone(mobmap_currentlyshown.zonename)==false) then 
		if(showed) then HideUIPanel(WorldMapFrame); end
		return; 
	end
	if(mobmap_currentlyshown.posdata==nil) then MobMap_UpdatePositions(); end
	MobMapButtonFrameCurrentMob:SetText(mobmap_currentlyshown.mobname);
	MobMap_HideAllDots();
	MobMap_DisplayPositionData(mobmap_currentlyshown.posdata, mobmap_currentlyshown.mobid);
end

mobmap_displayed_dot_count = 0;

function MobMap_DisplayPositionData(posdata, mobid)
	if(posdata==nil) then return; end
	for k,v in pairs(posdata) do
		for x=v.x1,v.x2,1 do
			local frame=getglobal("MobMapDot"..x.."_"..v.y);
			if(frame==nil) then 
				frame=CreateFrame("Frame","MobMapDot"..x.."_"..v.y,WorldMapPositioningGuide,"MobMapDotFrameTemplate");
				frame:SetPoint("TOPLEFT",WorldMapPositioningGuide,"TOPLEFT",x*frame:GetWidth(),-67-v.y*frame:GetHeight());
				frame:SetFrameLevel(WorldMapPositioningGuide:GetFrameLevel()+21);
				frame:SetAlpha(mobmap_dot_transparency);
				frame.xcoord=x;
				frame.ycoord=v.y;
			end
			if(mobid~=nil) then
				if(frame.idtable==nil) then
					frame.idtable={};
				end
				table.insert(frame.idtable,mobid);
			end
			mobmap_displayed_dot_count=mobmap_displayed_dot_count+1;
			frame:Show();
		end
	end
	MobMap_RescalePositionDots();
end

function MobMap_HideAllDots()
	mobmap_displayed_dot_count=0;
	for x=1,99,1 do
		for y=1,99,1 do
			local frame=getglobal("MobMapDot"..x.."_"..y);
			if(frame~=nil) then 
				frame:Hide();
				frame.idtable=nil;
			end
			local mapiconframe=getglobal("MobMapMinimapDot"..x.."_"..y);
			if(mapiconframe~=nil) then
				mobmap_astrolabe_library:RemoveIconFromMinimap(mapiconframe);
			end
		end
	end
end

function MobMap_HideAllMinimapDots()
	for x=1,99,1 do
		for y=1,99,1 do
			local mapiconframe=getglobal("MobMapMinimapDot"..x.."_"..y);
			if(mapiconframe~=nil) then
				mobmap_astrolabe_library:RemoveIconFromMinimap(mapiconframe);
			end
		end
	end
end

function MobMap_DisplayDotTooltip()
	if(this.idtable~=nil) then
		WorldMapTooltip:SetOwner(this, "ANCHOR_TOPLEFT");
		WorldMapTooltip:AddDoubleLine("Position:",this.xcoord..","..this.ycoord,1,1,1,1,1,1);
		local k,v;
		for k,v in pairs(this.idtable) do
			WorldMapTooltip:AddLine(MobMap_GetMobName(v));
		end		
		WorldMapTooltip:Show();
	end
end

function MobMap_HideDotTooltip()
	WorldMapTooltip:Hide();
end

function MobMap_UpdateMinimapIcons()
	if(mobmap_currentlyshown==nil) then return; end
	if(mobmap_astrolabe_library==nil) then
		mobmap_astrolabe_library=DongleStub(mobmap_astrolabe_version);
	end
	local continent,zone,playerx,playery=mobmap_astrolabe_library:GetCurrentPlayerPosition();
	if(playerx==nil or playery==nil) then return; end
	if(MobMap_GetCurrentMapZoneName()~=mobmap_currentlyshown.zonename) then return; end
	for k,v in pairs(mobmap_currentlyshown.posdata) do
		for x=v.x1,v.x2,1 do
			local mapiconframe=getglobal("MobMapMinimapDot"..x.."_"..v.y);
			if(mapiconframe==nil) then
				mapiconframe=CreateFrame("Frame","MobMapMinimapDot"..x.."_"..v.y,Minimap,"MobMapDotFrameTemplate");
				mapiconframe.isMinimapButton=true;
			end
			if(MobMap_GetDistance(playerx,playery,x/100,v.y/100)<0.04) then
				mobmap_astrolabe_library:PlaceIconOnMinimap(mapiconframe, continent, zone, x/100, v.y/100);
			else
				mobmap_astrolabe_library:RemoveIconFromMinimap(mapiconframe);
			end
		end
	end
end

function MobMap_GetDistance(x1,y1,x2,y2)
	return math.sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
end

function MobMap_ShowMobByID(mobid)
	local mobname=MobMap_GetMobName(mobid);
	if(mobname==nil) then return; end
	MobMap_ShowMobByName(mobname);
end

function MobMap_ShowMobByName(mobname)
	MobMapMobSearchFrameSearchBox:SetText("\""..mobname.."\"");
	MobMapMobSearchFrameSubtitleSearchBox:SetText("");
	MobMap_UpdateFilter(MobMapMobSearchFrameSearchBox:GetText(),MobMapMobSearchFrameSubtitleSearchBox:GetText());
end

function MobMap_ShowMultipleMobs(idlist)
	local searchtext="id:";
	for i=1,table.getn(idlist),1 do
		searchtext=searchtext..idlist[i]..",";
	end
	MobMapMobSearchFrameSearchBox:SetText(searchtext);
	MobMapMobSearchFrameSubtitleSearchBox:SetText("");
	MobMap_UpdateFilter(MobMapMobSearchFrameSearchBox:GetText(),MobMapMobSearchFrameSubtitleSearchBox:GetText());
end

-- MobMapMobSearchFrame event handlers

mobmap_mobsearchframe_oldtext1="";
mobmap_mobsearchframe_oldtext2="";
mobmap_mobsearchframe_timeout=0;

function MobMapMobSearchFrame_OnShow()
	mobmap_mobsearchframe_oldtext1=MobMapMobSearchFrameSearchBox:GetText();
	mobmap_mobsearchframe_oldtext2=MobMapMobSearchFrameSubtitleSearchBox:GetText();
	mobmap_mobsearchframe_timeout=-1;
end

function MobMapMobSearchFrame_OnUpdate()
	if(MobMapMobSearchFrameSearchBox:GetText()~=mobmap_mobsearchframe_oldtext1 or MobMapMobSearchFrameSubtitleSearchBox:GetText()~=mobmap_mobsearchframe_oldtext2) then
		mobmap_mobsearchframe_oldtext1=MobMapMobSearchFrameSearchBox:GetText();
		mobmap_mobsearchframe_oldtext2=MobMapMobSearchFrameSubtitleSearchBox:GetText();
		mobmap_mobsearchframe_timeout=0.5;
	end
	if(mobmap_mobsearchframe_timeout==-1) then return; end
	if(mobmap_mobsearchframe_timeout<0) then
		MobMap_UpdateFilter(MobMapMobSearchFrameSearchBox:GetText(),MobMapMobSearchFrameSubtitleSearchBox:GetText());
		mobmap_mobsearchframe_timeout=-1;
	else
		mobmap_mobsearchframe_timeout=mobmap_mobsearchframe_timeout-arg1;
	end
end